@@ -3,23 +3,21 @@
 
 from __future__ import annotations
 
-import subprocess
+import sys
 import zlib
+from collections.abc import Sequence
 from contextlib import contextmanager
 from io import BytesIO
 from math import ceil
 from os import fspath
 from pathlib import Path
 from subprocess import run
-from typing import NamedTuple, Sequence
+from typing import NamedTuple
 
-import deprecation
 import PIL
 import pytest
-from conftest import needs_python_v
 from hypothesis import assume, given, note, settings
 from hypothesis import strategies as st
-from packaging.version import Version
 from PIL import Image, ImageChops, ImageCms
 from PIL import features as PIL_features
 
@@ -28,7 +26,6 @@ from pikepdf import (
     Array,
     Dictionary,
     Name,
-    Object,
     Operator,
     Pdf,
     PdfError,
@@ -40,12 +37,20 @@ from pikepdf import (
 )
 from pikepdf.models._transcoding import _next_multiple, unpack_subbyte_pixels
 from pikepdf.models.image import (
-    DependencyError,
-    NotExtractableError,
     PdfJpxImage,
     UnsupportedImageTypeError,
 )
 
+if sys.version_info.releaselevel in ('alpha', 'beta'):
+    # When testing a pre-release, we are likely building Pillow from source, and it will
+    # be missing several of its libraries and trigger errors around missing libtiff and
+    # the CMS library. Rather than trying to get a full build of Pillow, just skip these
+    # tests.
+    pytest.skip(
+        "skipping image tests on alpha/beta due to complex Pillow deps",
+        allow_module_level=True,
+    )
+
 # pylint: disable=redefined-outer-name
 
 
@@ -92,11 +97,6 @@ def sandwich(first_image_in):
     return first_image_in('sandwich.pdf')
 
 
-@pytest.fixture
-def jbig2(first_image_in):
-    return first_image_in('jbig2.pdf')
-
-
 @pytest.fixture
 def trivial(first_image_in):
     return first_image_in('pal-1bit-trivial.pdf')
@@ -209,13 +209,11 @@ def test_lowlevel_replace_jpeg(congress, outdir):
     pdf.save(outdir / 'congress_gray.pdf')
 
 
-@deprecation.fail_if_not_removed
 def test_inline(inline):
     iimage, pdf = inline
     assert iimage.width == 8
     assert not iimage.image_mask
     assert iimage.mode == 'RGB'
-    assert iimage.is_inline
     assert iimage.colorspace == '/DeviceRGB'
     assert 'PdfInlineImage' in repr(iimage)
 
@@ -493,11 +491,13 @@ def valid_random_palette_image_spec(
     channels = (
         1
         if colorspace == Name.DeviceGray
-        else 3
-        if colorspace == Name.DeviceRGB
-        else 4
-        if colorspace == Name.DeviceCMYK
-        else 0
+        else (
+            3
+            if colorspace == Name.DeviceRGB
+            else 4
+            if colorspace == Name.DeviceCMYK
+            else 0
+        )
     )
 
     if not palette:
@@ -508,7 +508,6 @@ def valid_random_palette_image_spec(
     return PaletteImageSpec(bpc, width, height, hival, colorspace, palette, imbytes)
 
 
-@deprecation.fail_if_not_removed
 @pytest.mark.parametrize(
     'filename,bpc,rgb',
     [
@@ -523,7 +522,6 @@ def test_image_palette(resources, filename, bpc, rgb):
 
     assert pim.palette[0] == 'RGB'
     assert pim.colorspace == '/DeviceRGB'
-    assert not pim.is_inline
     assert pim.mode == 'P'
     assert pim.bits_per_component == bpc
 
@@ -555,6 +553,7 @@ def first_image_from_pdfimages(pdf, tmpdir):
 
 
 @given(spec=valid_random_palette_image_spec())
+@settings(deadline=60000)
 def test_image_palette2(spec, tmp_path_factory):
     pdf = pdf_from_palette_image_spec(spec)
     pim = PdfImage(pdf.pages[0].Resources.XObject['/Im0'])
@@ -590,7 +589,6 @@ def test_bool_in_inline_image():
 @pytest.mark.skipif(
     not PIL_features.check_codec('jpg_2000'), reason='no JPEG2000 codec'
 )
-@deprecation.fail_if_not_removed
 def test_jp2(first_image_in):
     xobj, _pdf = first_image_in('pike-jp2.pdf')
     pim = PdfImage(xobj)
@@ -598,7 +596,6 @@ def test_jp2(first_image_in):
 
     assert '/JPXDecode' in pim.filters
     assert pim.colorspace == '/DeviceRGB'
-    assert not pim.is_inline
     assert not pim.indexed
     assert pim.mode == 'RGB'
     assert pim.bits_per_component == 8
@@ -635,15 +632,13 @@ def test_extract_direct_fails_nondefault_colortransform(congress):
     pim = PdfImage(xobj)
 
     bio = BytesIO()
-    with pytest.raises(NotExtractableError):
-        pim._extract_direct(stream=bio)
+    assert pim._extract_direct(stream=bio) is None
     with pytest.raises(UnsupportedImageTypeError):
         pim.extract_to(stream=bio)
 
     xobj.ColorSpace = Name.DeviceCMYK
     pim = PdfImage(xobj)
-    with pytest.raises(NotExtractableError):
-        pim._extract_direct(stream=bio)
+    assert pim._extract_direct(stream=bio) is None
     with pytest.raises(UnsupportedImageTypeError):
         pim.extract_to(stream=bio)
 
@@ -694,23 +689,32 @@ def test_stacked_compression(first_image_in):
     assert pim.filters == ['/FlateDecode', '/JPXDecode']
 
 
-def test_ccitt_photometry(sandwich):
+@pytest.mark.parametrize(
+    'blackis1,decode,expected',
+    [
+        (None, None, 255),
+        (False, None, 255),
+        (True, None, 0),
+        (None, [0, 1], 255),
+        (None, [1, 0], 0),
+        (False, [0, 1], 255),
+        (False, [1, 0], 0),
+        (True, [0, 1], 0),
+        (True, [1, 0], 255),
+    ],
+)
+def test_ccitt_photometry(sandwich, blackis1, decode, expected):
     xobj, _pdf = sandwich
 
+    if blackis1 is not None:
+        xobj.DecodeParms.BlackIs1 = blackis1
+    if decode is not None:
+        xobj.Decode = decode
+
     pim = PdfImage(xobj)
     im = pim.as_pil_image()
     im = im.convert('L')
-    assert im.getpixel((0, 0)) == 255, "Expected white background"
-
-    xobj.DecodeParms.BlackIs1 = True
-    im = pim.as_pil_image()
-    im = im.convert('L')
-    assert im.getpixel((0, 0)) == 255, "Expected white background"
-
-    xobj.DecodeParms.BlackIs1 = False
-    im = pim.as_pil_image()
-    im = im.convert('L')
-    assert im.getpixel((0, 0)) == 255, "Expected white background"
+    assert im.getpixel((0, 0)) == expected, f"Expected background pixel = {expected}"
 
 
 def test_ccitt_encodedbytealign(sandwich):
@@ -732,133 +736,6 @@ def test_imagemagick_uses_rle_compression(first_image_in):
     assert im.getpixel((5, 5)) == (255, 128, 0)
 
 
-# Unforuntately pytest cannot test for this using "with pytest.warns(...)".
-# Suppression is the best we can manage
-suppress_unraisable_jbigdec_error_warning = pytest.mark.filterwarnings(
-    "ignore:.*jbig2dec error.*:pytest.PytestUnraisableExceptionWarning"
-)
-
-
-@needs_python_v("3.8", reason="for pytest unraisable exception support")
-@suppress_unraisable_jbigdec_error_warning
-def test_jbig2_not_available(jbig2, monkeypatch):
-    xobj, _pdf = jbig2
-    pim = PdfImage(xobj)
-
-    class NotFoundJBIG2Decoder(pikepdf.jbig2.JBIG2DecoderInterface):
-        def check_available(self):
-            raise DependencyError('jbig2dec') from FileNotFoundError('jbig2dec')
-
-        def decode_jbig2(self, jbig2: bytes, jbig2_globals: bytes) -> bytes:
-            raise FileNotFoundError('jbig2dec')
-
-    monkeypatch.setattr(pikepdf.jbig2, 'get_decoder', NotFoundJBIG2Decoder)
-
-    assert not pikepdf.jbig2.get_decoder().available()
-
-    with pytest.raises(DependencyError):
-        pim.as_pil_image()
-
-
-needs_jbig2dec = pytest.mark.skipif(
-    not pikepdf.jbig2.get_decoder().available(), reason="jbig2dec not installed"
-)
-
-
-@needs_jbig2dec
-def test_jbig2_extractor(jbig2):
-    xobj, _pdf = jbig2
-    pikepdf.jbig2.get_decoder().decode_jbig2(xobj.read_raw_bytes(), b'')
-
-
-@needs_jbig2dec
-def test_jbig2(jbig2):
-    xobj, _pdf = jbig2
-    pim = PdfImage(xobj)
-    im = pim.as_pil_image()
-    assert im.size == (1000, 1520)
-    assert im.getpixel((0, 0)) == 0  # Ensure loaded
-
-
-@needs_jbig2dec
-def test_jbig2_decodeparms_null_issue317(jbig2):
-    xobj, _pdf = jbig2
-    xobj.stream_dict = Object.parse(
-        b'''<< /BitsPerComponent 1
-               /ColorSpace /DeviceGray
-               /Filter [ /JBIG2Decode ]
-               /DecodeParms null
-               /Height 1520
-               /Length 19350
-               /Subtype /Image
-               /Type /XObject
-               /Width 1000
-            >>'''
-    )
-    pim = PdfImage(xobj)
-    im = pim.as_pil_image()
-    assert im.size == (1000, 1520)
-    assert im.getpixel((0, 0)) == 0  # Ensure loaded
-
-
-@needs_jbig2dec
-def test_jbig2_global(first_image_in):
-    xobj, _pdf = first_image_in('jbig2global.pdf')
-    pim = PdfImage(xobj)
-    im = pim.as_pil_image()
-    assert im.size == (4000, 2864)
-    assert im.getpixel((0, 0)) == 255  # Ensure loaded
-
-
-@needs_jbig2dec
-def test_jbig2_global_palette(first_image_in):
-    xobj, _pdf = first_image_in('jbig2global.pdf')
-    xobj.ColorSpace = pikepdf.Array(
-        [Name.Indexed, Name.DeviceRGB, 1, b'\x00\x00\x00\xff\xff\xff']
-    )
-    pim = PdfImage(xobj)
-    im = pim.as_pil_image()
-    assert im.size == (4000, 2864)
-    assert im.getpixel((0, 0)) == 255  # Ensure loaded
-
-
-@needs_python_v("3.8", reason="for pytest unraisable exception support")
-@suppress_unraisable_jbigdec_error_warning
-def test_jbig2_error(first_image_in, monkeypatch):
-    xobj, _pdf = first_image_in('jbig2global.pdf')
-    pim = PdfImage(xobj)
-
-    class BrokenJBIG2Decoder(pikepdf.jbig2.JBIG2DecoderInterface):
-        def check_available(self):
-            return
-
-        def decode_jbig2(self, jbig2: bytes, jbig2_globals: bytes) -> bytes:
-            raise subprocess.CalledProcessError(1, 'jbig2dec')
-
-    monkeypatch.setattr(pikepdf.jbig2, 'get_decoder', BrokenJBIG2Decoder)
-
-    pim = PdfImage(xobj)
-    with pytest.raises(PdfError, match="unfilterable stream"):
-        pim.as_pil_image()
-
-
-@needs_python_v("3.8", reason="for pytest unraisable exception support")
-@suppress_unraisable_jbigdec_error_warning
-def test_jbig2_too_old(first_image_in, monkeypatch):
-    xobj, _pdf = first_image_in('jbig2global.pdf')
-    pim = PdfImage(xobj)
-
-    class OldJBIG2Decoder(pikepdf.jbig2.JBIG2Decoder):
-        def _version(self):
-            return Version('0.12')
-
-    monkeypatch.setattr(pikepdf.jbig2, 'get_decoder', OldJBIG2Decoder)
-
-    pim = PdfImage(xobj)
-    with pytest.raises(DependencyError, match='too old'):
-        pim.as_pil_image()
-
-
 def test_ccitt_icc(first_image_in, resources):
     xobj, pdf = first_image_in('sandwich.pdf')
 
@@ -933,6 +810,8 @@ CMYK_PINK = b'\x04\xc0\x00\x15'
 
 CMYK_PALETTE = CMYK_RED + CMYK_GREEN + CMYK_BLUE + CMYK_PINK
 
+GRAY_RGB_PALETTE = b''.join(bytes([gray, gray, gray]) for gray in range(256))
+
 
 @pytest.mark.parametrize(
     'base, hival, bits, palette, expect_type, expect_mode',
@@ -940,6 +819,14 @@ CMYK_PALETTE = CMYK_RED + CMYK_GREEN + CMYK_BLUE + CMYK_PINK
         (Name.DeviceGray, 4, 8, b'\x00\x40\x80\xff', 'L', 'P'),
         (Name.DeviceCMYK, 4, 8, CMYK_PALETTE, 'CMYK', 'P'),
         (Name.DeviceGray, 4, 4, b'\x04\x08\x02\x0f', 'L', 'P'),
+        (
+            Array([Name.CalRGB, Dictionary(WhitePoint=Array([1.0, 1.0, 1.0]))]),
+            255,
+            8,
+            GRAY_RGB_PALETTE,
+            'RGB',
+            'P',
+        ),
     ],
 )
 def test_palette_nonrgb(base, hival, bits, palette, expect_type, expect_mode):
@@ -1140,6 +1027,7 @@ def test_devicen():
         heights=st.integers(1, 7),
     )
 )
+@settings(deadline=None)
 def test_grayscale_stride(spec):
     pdf = pdf_from_image_spec(spec)
     pim = PdfImage(pdf.pages[0].Resources.XObject.Im0)
@@ -1239,3 +1127,76 @@ def test_random_image(spec, tmp_path_factory):
     #     diff.save('imdiff.png')
     #     breakpoint()
     #     assert False
+
+
+class StencilMaskSpec(NamedTuple):
+    width: int
+    height: int
+    imbytes: bytes
+
+    def to_pdf(self):
+        pdf = pikepdf.new()
+        pdfw, pdfh = 36 * self.width, 36 * self.height
+
+        pdf.add_blank_page(page_size=(pdfw, pdfh))
+
+        imobj = Stream(
+            pdf,
+            self.imbytes,
+            Width=self.width,
+            Height=self.height,
+            Type=Name.XObject,
+            Subtype=Name.Image,
+            ImageMask=True,
+        )
+
+        pdf.pages[0].Contents = Stream(
+            pdf, b'%f 0 0 %f 0 0 cm 0.5 0.75 1.0 rg /Im0 Do' % (pdfw, pdfh)
+        )
+        pdf.pages[0].Resources = Dictionary(XObject=Dictionary(Im0=imobj))
+        pdf.pages[0].MediaBox = Array([0, 0, pdfw, pdfh])
+        return pdf
+
+
+@st.composite
+def valid_random_stencil_mask_spec(
+    draw,
+    widths=st.integers(min_value=1, max_value=16),
+    heights=st.integers(min_value=1, max_value=16),
+):
+    width = draw(widths)
+    height = draw(heights)
+
+    min_imbytes = _next_multiple(width, 8) * height // 8
+    imbytes = draw(st.binary(min_size=min_imbytes, max_size=min_imbytes))
+
+    return StencilMaskSpec(width, height, imbytes)
+
+
+@given(spec=valid_random_stencil_mask_spec())
+def test_extract_stencil_mask(spec):
+    pdf = spec.to_pdf()
+    pim = PdfImage(pdf.pages[0].Resources.XObject.Im0)
+    bio = BytesIO()
+    pim.extract_to(stream=bio)
+    im = Image.open(bio)
+    assert im.mode == '1'
+
+
+def test_repr_when_mode_not_impl():
+    pdf = Pdf.new()
+    pim = PdfImage(
+        Stream(
+            pdf,
+            b'',
+            BitsPerComponent=1,
+            ColorSpace=Name.InvalidColorSpace,
+            Width=1,
+            Height=1,
+            Type=Name.XObject,
+            Subtype=Name.Image,
+        )
+    )
+    assert repr(pim).startswith('<pikepdf.PdfImage image mode=? size=1x1')
+    with pytest.raises(NotImplementedError):
+        pim.mode
